@@ -47,8 +47,10 @@ import           Data.ListLike.String
 import           Data.ListLike.IO
 import           Data.ListLike.FoldableLL
 import           Data.ListLike.Text ()
+import           Data.ListLike.UTF8
 import           Data.ListLike.Vector ()
 import           Data.Int
+import           Data.Maybe (fromMaybe)
 import           Data.Monoid
 import qualified Data.ByteString as BS
 import qualified Data.ByteString.Char8 as BSC
@@ -58,6 +60,8 @@ import qualified Data.Array.IArray as A
 import           Data.Array.IArray((!), (//), Ix(..))
 import qualified Data.ByteString.Lazy as BSL
 import qualified Data.ByteString.Lazy.Char8 as BSLC
+import           Data.String.UTF8 (UTF8)
+import qualified Data.String.UTF8 as UTF8
 import qualified System.IO as IO
 import           Data.Word
 
@@ -108,6 +112,7 @@ instance ListLike BS.ByteString Word8 where
     cons = BS.cons
     snoc = BS.snoc
     append = BS.append
+    uncons = BS.uncons
     head = BS.head
     last = BS.last
     tail = BS.tail
@@ -162,7 +167,7 @@ instance ListLike BS.ByteString Word8 where
     --insert = BS.insert
     toList = BS.unpack
     fromList = BS.pack
-    fromListLike = fromList . toList
+    --fromListLike = fromList . toList
     --nubBy = BS.nubBy
     --deleteBy = BS.deleteBy
     --deleteFirstsBy = BS.deleteFirstsBy
@@ -218,6 +223,7 @@ instance ListLike BSL.ByteString Word8 where
     cons = BSL.cons
     snoc = BSL.snoc
     append = BSL.append
+    uncons = BSL.uncons
     head = BSL.head
     last = BSL.last
     tail = BSL.tail
@@ -273,7 +279,7 @@ instance ListLike BSL.ByteString Word8 where
     --insert = BSL.insert
     toList = BSL.unpack
     fromList = BSL.pack
-    fromListLike = fromList . toList
+    --fromListLike = fromList . toList
     --nubBy = BSL.nubBy
     --deleteBy = BSL.deleteBy
     --deleteFirstsBy = BSL.deleteFirstsBy
@@ -547,7 +553,7 @@ instance ListLike (S.Seq a) a where
     --insert = S.insert
     toList = F.toList
     fromList = S.fromList
-    fromListLike = fromList . toList
+    --fromListLike = fromList . toList
     --nubBy =
     --deleteBy =
     --deleteFirstsBy =
